#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "error.c"

#ifdef __GLIBC__
#define __NO_STAT64
#endif

static char* name="ln: ";

void oops(const char *fn) {
  write(2,name,strlen(name));
  if (fn) { write(2,fn,strlen(fn)); write(2,": ",2); }
  switch (errno) {
  case EPERM:		warn("permission denied"); break;
  case EEXIST:		warn("file exists"); break;
  case EFAULT:		warn("invalid pointer"); break;
  case ENAMETOOLONG:	warn("name too long"); break;
  case ENOENT:		warn("file not found"); break;
  case ENOMEM:		warn("out of virtual memory"); break;
  case EROFS:		warn("read-only file system"); break;
  case EACCES:		warn("directory search permissions denied"); break;
  case ELOOP:		warn("too many symbolic links"); break;
  case ENOSPC:		warn("no space left on device/quota exceeded"); break;
  case EIO:		warn("I/O error"); break;
  default: warn("unknown error");
  }
}

void panic(const char *fn) {
  oops(fn);
  exit(1);
}

unsigned int str_copy(char *out,const char *in) {
  register char* s=out;
  register const char* t=in;
  for (;;) {
    if (!(*s=*t)) break; ++s; ++t;
    if (!(*s=*t)) break; ++s; ++t;
    if (!(*s=*t)) break; ++s; ++t;
    if (!(*s=*t)) break; ++s; ++t;
  }
  return s-out;
}

unsigned int str_rchr(const char *in, char needle) {
  register const char* t=in;
  register const char c=needle;
  register const char* found=0;
  for (;;) {
    if (!*t) break; if (*t==c) found=t; ++t;
    if (!*t) break; if (*t==c) found=t; ++t;
    if (!*t) break; if (*t==c) found=t; ++t;
    if (!*t) break; if (*t==c) found=t; ++t;
  }
  return (found?found:t)-in;
}

int f=0,s=0;

int dolink(const char *src,const char *dest,int mustbedir) {
  char *d=(char*)dest;
  int idx=str_rchr(src,'/');
  const char *f=src[idx]?src+idx+1:src;
#ifdef __NO_STAT64
  struct stat ss;
  if (stat(dest,&ss))
#else
  struct stat64 ss;
  if (stat64(dest,&ss))
#endif
  {
    if (mustbedir)
      error("ln: when making multiple links, last argument must be a directory");
    if (s)
      return symlink(src,dest);
    else
      return link(src,dest);
  } else {
    if (S_ISDIR(ss.st_mode)) {
      char *e;
      d=alloca(strlen(dest)+strlen(f)+2);
      e=d+str_copy(d,dest);
      *e='/'; ++e;
      e+=str_copy(e,f);
      *e=0;
/*      printf("%slink %s to %s\n",s?"sym":"",src,d); */
      return dolink(src,d,0);
    }
    if (f)
      unlink(dest);
  }
}

main(int argc,char *argv[]) {
  int i;
  char *tmp;
  if (argc<2) {
    puts("usage: ln [-fs] source... destination");
    exit(0);
  }
  if (argv[1][0]=='-') {
    int j;
    for (j=1; j<strlen(argv[1]); j++) {
      switch (argv[1][j]) {
      case 'f': f=1; break;
      case 's': s=1; break;
      }
    }
    i=2;
  } else
    i=1;
  switch (argc-i) {
  case 1:
    tmp=strrchr(argv[i],'/');
    if (tmp) ++tmp; else tmp=argv[i];
    if (dolink(argv[i],tmp,0))
      oops(argv[i]);
    break;
  case 2:
    if (dolink(argv[i],argv[i+1],0))
      oops(argv[i]);
    break;
  default:
    while (i<argc-1) {
      if (dolink(argv[i],argv[argc-1],1))
	oops(argv[i]);
      ++i;
    }
  }
  return 0;
}

