#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>

static void error(char *message) {
  write(2,"mknod: ",6);
  write(2,message,strlen(message));
  write(2,"\n",1);
  exit(1);
}

#include "parsemode.c"

void panic() {
  switch (errno) {
  case EPERM:
    error("permission denied");
  case EEXIST:
    error("file exists");
  case EFAULT:
    error("invalid pointer");
  case ENAMETOOLONG:
    error("name too long");
  case ENOENT:
    error("file not found");
  case ENOMEM:
    error("out of virtual memory");
  case EROFS:
    error("read-only file system");
  case EACCES:
    error("directory search permissions denied");
  case ELOOP:
    error("too many symbolic links");
  case ENOSPC:
    error("no space left on device/quota exceeded");
  case EIO:
    error("I/O error");
  default:
    error("unknown error");
  }
}

void domknod(char *s,int m,int v,int iexist,char *full) {
  errno=0;
  if (mkdir(s,m))
    if (!iexist || errno!=EEXIST) panic();
  if (errno==0) {
    if (v) {
      write(2,"mkdir: created directory `",26);
      write(2,full,strlen(full));
      write(2,"'\n",2);
    }
  }
}

void usage() {
  error("usage: mknod [-m mode] filename type major minor\ntype == {b|c|u|p}");
}

main(int argc,char *argv[]) {
  int i;
  int mode=0666-umask(0);
  mode_t and,or;
  unsigned int count=0;
  char *name=0;
  char type=0;
  unsigned short major=0,minor=0;
  dev_t dev;
  for (i=1; i<argc; i++) {
    if (!argv[i]) continue;
    if (argv[i][0]=='-') {
      int j;
      for (j=1; j<strlen(argv[i]); j++) {
	switch (argv[i][j]) {
	case 'm': parsemode(argv[i+1],&and,&or); mode=(mode&and)|or; argv[i+1]=0; break;
	}
      }
    } else {
      switch (++count) {
      case 1: name=argv[i]; break;
      case 2: if (strlen(argv[i])!=1) error("[bcup] expected"); type=argv[i][0]; break;
      case 3: major=atoi(argv[i]); break;
      case 4: minor=atoi(argv[i]); break;
      case 5: usage();
      }
    }
  }
  if (type=='p') count+=2;
  if (count<4) usage();
  dev=makedev(major,minor);
  switch (type) {
  case 'b': mode|=S_IFBLK; break;
  case 'c': case 'u': mode|=S_IFCHR; break;
  case 'p': mode|=S_IFIFO; break;
  }
  if (mknod(name,mode,dev)) panic();
  return 0;
}

